// Connect SDA pin to A4 and SLC pin to A5, with input voltage of 5V

#include "Adafruit_MPU6050.h" // https://github.com/hideakitai/MPU9250 (Made by Hideakitai)

Adafruit_MPU6050 mpu;


void setup() {
    Serial.begin(38400);
    Wire.begin();
    delay(2000);

    if (!mpu.begin(0x68)) {  // change to your own address
        while (1) {
            Serial.println(F("MPU connection failed. Please check your connection with `connection_check` example."));
            delay(5000);
        }
    }

    // calibrate anytime you want to

    
    Serial.println(F("Accel Gyro calibration will start in 2sec."));
    Serial.println(F("Please leave the device still on the flat plane."));

    delay(2000);
    mpu.calibrateAccelGyro();
    
    Serial.println(F("Mag calibration will start in 2sec."));
    Serial.println(F("Please Wave device in a figure eight until done."));
    delay(2000);
    Serial.println(F("START!"));
    //Figure 8 motion is very important to get right, it really affects the readings
    mpu.calibrateMag();

    Serial.println("< calibration parameters >");
    Serial.println("accel bias [g]: ");
    Serial.print(mpu.getAccBiasX() * 1000.f / (float)MPU9250::CALIB_ACCEL_SENSITIVITY);
    Serial.print(", ");
    Serial.print(mpu.getAccBiasY() * 1000.f / (float)MPU9250::CALIB_ACCEL_SENSITIVITY);
    Serial.print(", ");
    Serial.print(mpu.getAccBiasZ() * 1000.f / (float)MPU9250::CALIB_ACCEL_SENSITIVITY);
    Serial.println();
    Serial.println("gyro bias [deg/s]: ");
    Serial.print(mpu.getGyroBiasX() / (float)MPU9250::CALIB_GYRO_SENSITIVITY);
    Serial.print(", ");
    Serial.print(mpu.getGyroBiasY() / (float)MPU9250::CALIB_GYRO_SENSITIVITY);
    Serial.print(", ");
    Serial.print(mpu.getGyroBiasZ() / (float)MPU9250::CALIB_GYRO_SENSITIVITY);
    Serial.println();
    Serial.println("mag bias [mG]: ");
    Serial.print(mpu.getMagBiasX());
    Serial.print(", ");
    Serial.print(mpu.getMagBiasY());
    Serial.print(", ");
    Serial.print(mpu.getMagBiasZ());
    Serial.println();
    Serial.println("mag scale []: ");
    Serial.print(mpu.getMagScaleX());
    Serial.print(", ");
    Serial.print(mpu.getMagScaleY());
    Serial.print(", ");
    Serial.print(mpu.getMagScaleZ());
    Serial.println();

    /*
    mpu.setAccBias(-27.29, -982.92, 942.88);
    mpu.setGyroBias(0.48, 0.48, 0.50);
    mpu.setMagBias(246.68, -2.67, -350);
    mpu.setMagScale(1.023, 1.202, 0.914);
    mpu.setMagneticDeclination(0.11);
    -----------------
    */
    //Set Gyro and Accel Bias
    //mpu.setAccBias(1.16, 24.36, -4.44);
    //mpu.setGyroBias(-2.09, 0.27, 1.76);
    // More Mag Calibrations
    //mpu.setMagBias(209.41, 279.54, -562.88);
    //mpu.setMagScale(1.03, 1.13, 0.87);    
      // Good Mag Calibrations
    //mpu.setMagBias(209.588, 250.525, -311.644);
    //mpu.setMagScale(0.90, 0.888, 1.6);
    

        //select filter for IMU data and number of iterations
    mpu.selectFilter(QuatFilterSel {QuatFilterSel::MAHONY});
    mpu.setFilterIterations(20); // 10 to 20 is good for stable yaw estimation
}

void loop() {
    if (mpu.update()) {
        static uint32_t prev_ms = millis();
        if (millis() > prev_ms + 25) {
            print_X_Y_Z();
            prev_ms = millis();
        }
    }
}
/*
void print_X_Y_Z() {
    Serial.print("X, Y, Z: ");
    Serial.print(mpu.getEulerX(), 2);
    Serial.print(", ");
    Serial.print(mpu.getEulerY(), 2);
    Serial.print(", ");
    Serial.println(mpu.getEulerZ(), 2);
}
*/
void print_X_Y_Z() {
    Serial.print("X: ");
    Serial.print(mpu.getEulerX(), 2);
    Serial.print("°, Y: ");
    Serial.print(mpu.getEulerY(), 2);
    Serial.print("°, Z: ");
    Serial.print(mpu.getEulerZ(), 2);
    Serial.println("°");
}
